@@ -53,6 +53,7 @@ typedef enum {
     text_fuzzy_status_open_error,
     text_fuzzy_status_close_error,
     text_fuzzy_status_read_error,
+    text_fuzzy_status_line_too_long,
 }
 text_fuzzy_status_t;
 
@@ -89,9 +90,9 @@ typedef struct text_fuzzy {
 }
 text_fuzzy_t;
 #line 47 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
-text_fuzzy_status_t text_fuzzy_compare_single (text_fuzzy_t * text_fuzzy, text_fuzzy_string_t * b);
-#line 149 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
+text_fuzzy_status_t text_fuzzy_compare_single (text_fuzzy_t * tf, text_fuzzy_string_t * b);
+#line 158 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
 text_fuzzy_status_t text_fuzzy_set_search_term (text_fuzzy_t * text_fuzzy);
-#line 159 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
+#line 240 "/usr/home/ben/projects/Text-Fuzzy/text-fuzzy.c.in"
 text_fuzzy_status_t text_fuzzy_scan_file (text_fuzzy_t * text_fuzzy, char * file_name, char ** nearest_ptr);
 #endif /* TEXT_FUZZY_H */
